
import QtQuick 2.0
import MuseScore 3.0

// For this test we'll walk the entire first staff looking for chords.
// For each chord found, the first note will be removed, unless only one
// note remains. In that case a new note with pitch 'A' is added to the chord
// bringing its size up to two.
//
// Kind of silly but it gives undo something to do.

MuseScore {
   menuPath: "Plugins.Chord Add-Remove Test"
   description: "Test Chord.add and Chord.remove methods."
   version: "2.0"

   // For diagnostic use.
   function dumpObjectEntries(obj, showUndefinedVals, title) {
      console.log("VV -------- " + title + " ---------- VV")
      for (let [key, value] of Object.entries(obj)) {
         if (showUndefinedVals || (value != null)) {
            console.log(key + "=" + value)
         }
      }
      console.log("^^ -------- " + title + " ---------- ^^")
   }

   function dumpNotes(chord) {
      for (var idx = 0; idx < chord.notes.length; idx++) {
         var note = chord.notes[idx];
         console.log("note[" + idx + "].pitch=" + note.pitch)
      }
   }

   function chord_add_remove_test()
   {
      if (typeof curScore === 'undefined')
         return
      // We need a cursor
      var cursor = curScore.newCursor();

      console.log("Scanning all chords in the first staff of the score...")

      cursor.rewind(0)        // Start at beginning of score
      cursor.staffIdx = 0     // Focus on first staff
      // Toggled between Chord.remove() and removeElement()
      var toggleRemoveImplementation = true;
      while (cursor.segment) {
         // Looking only for chords
         if (cursor.element && cursor.element.type == Element.CHORD) {
            console.log("=============")
            console.log("Found a Chord")
            var chord = cursor.element;

            console.log("Number of notes in chord before test = " + chord.notes.length + ", They are:")
            dumpNotes(chord)
            if (chord.notes.length <= 1) {
               // Lets add a note back so the chord isn't a dead end.
               console.log("TEST: Call chord.add(newNote)...")
               var newNote = newElement(Element.NOTE);
               newNote.pitch = 69      // Add an A note
               chord.add(newNote)
            } else {
               // Removes first note located at the bottom of the chord.
               if (toggleRemoveImplementation) {
                  console.log("TEST: Call chord.remove(chord.notes[0])...")
                  chord.remove(chord.notes[0])
               }
               else {
                  console.log("TEST: Call removeElement(chord.notes[0])...")
                  removeElement(chord.notes[0])
               }
               toggleRemoveImplementation = !toggleRemoveImplementation;
            }
            console.log("Number of notes in chord after test = " + chord.notes.length + ", They are:")
            dumpNotes(chord)
            console.log("-------------")
         }
         // Step to the next element.
         cursor.next();
      }
   }

   onRun: {
      console.log(description)

      chord_add_remove_test()

      console.log("Tests completed.")

      Qt.quit()
   }
}
